#!/usr/bin/python

"""
Timing analysis code based on analyzer from old bootspeed scripts.
"""
import datetime
import json
import netifaces
import os
import platform
import re
import socket
import subprocess
import sys

from utah.client.common import get_media_info


def getHwAddr():
    for iface in netifaces.interfaces():
        if iface != 'lo' and 'wlan' not in iface:
            return netifaces.ifaddresses(iface)[netifaces.AF_LINK][0]['addr']


uuid = open(os.path.join('/etc', 'utah', 'uuid'), 'r').read().strip()
dir = '/var/lib/utah-bootspeed'

timings = [0.0, 0.0, 0.0, 0.0]

timesfile = os.path.join(dir, 'times')
with open(timesfile, 'r') as t:
    times = t.read().split("\n")
for i in range(0, 4):
    try:
        timings[i] = float(times[i]) / 100
    except ValueError:
        pass

kernelinit = float(0)
dmesgfile = os.path.join(dir, 'dmesg')
with open(dmesgfile, 'r') as d:
    for line in d.readlines():
        match = re.search('^\[\s*(\d+\.\d+)\] Write protecting the kernel',
                          line)
        if match is not None:
            kernelinit = float(match.group(1))
            break

filename = os.path.join('/', 'etc', 'utah', 'md5')
md5 = 'unknown'
if os.path.exists(filename) and os.access(filename, os.R_OK):
    with open(filename, 'r') as f:
        md5 = f.read().strip()

filename = os.path.join('/', 'etc', 'utah', 'install-type')
variant = 'unknown'
if os.path.exists(filename) and os.access(filename, os.R_OK):
    with open(filename, 'r') as f:
        variant = f.read().strip()

test_info = {
    "machine_name": socket.gethostname(),
    "machine_mac": getHwAddr(),
    "media-info": get_media_info(),
    "image_build_number": get_media_info().split()[-1].strip('()'),
    "image_arch": subprocess.check_output(['dpkg-architecture',
                                           '-qDEB_HOST_ARCH']).strip(),
    "image_md5": md5,
    "image_release": platform.linux_distribution()[2],
    "image_variant": variant,
    "kernel_init": kernelinit,
    "kernel": timings[1],
    "plumbing": timings[2] - timings[1],
    "xorg": timings[3] - timings[2],
    "desktop": timings[0] - timings[3],
    "boot": timings[0],
    "ran_at": datetime.datetime.now().isoformat(),
}

infofile = open(os.path.join(dir, 'boot.json'), 'w')

json.dump(test_info, infofile, indent=4, sort_keys=True)

# If we have a zero timing, our test has failed
for stage in timings:
    if stage == 0.0:
        sys.exit(1)
